"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PreProcessorExecutor = void 0;
const the_lodash_1 = __importDefault(require("the-lodash"));
const the_promise_1 = require("the-promise");
const child_process_1 = require("child_process");
const path_1 = __importDefault(require("path"));
const input_source_1 = require("../input/input-source");
const spinner_1 = require("../screen/spinner");
const yaml_1 = __importDefault(require("yaml"));
const fs_1 = __importDefault(require("fs"));
class PreProcessorExecutor {
    constructor(logger, manifestsLoader) {
        this._logger = logger.sublogger("PreProcessorExecutor");
        this._manifestsLoader = manifestsLoader;
        this._manifestPackage = manifestsLoader.manifestPackage;
    }
    execute(inputSource, source) {
        return __awaiter(this, void 0, void 0, function* () {
            if (inputSource.preprocessor === 'kustomize') {
                yield this._kustomize(inputSource, source);
            }
            if (inputSource.preprocessor === 'helm') {
                yield this._helm(inputSource, source);
            }
        });
    }
    _kustomize(inputSource, source) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const dirName = path_1.default.dirname(inputSource.path);
            source = source.getSource("kustomize", dirName, source.originalSource);
            try {
                const command = `kustomize build ${dirName}`;
                const contents = yield yield this.executeCommand(command);
                if (!contents) {
                    source.reportError(`Could not extract manifest contents from ${inputSource.preprocessor}`);
                }
                else {
                    this._manifestsLoader.parseContents(source, "manifests.yaml", contents);
                }
            }
            catch (reason) {
                this._logger.info("[_loadFile] ERROR: ", reason);
                source.reportError(`Failed to extract manifest from ${inputSource.preprocessor}. Reason: ${(_a = reason === null || reason === void 0 ? void 0 : reason.message) !== null && _a !== void 0 ? _a : "Unknown"}`);
            }
        });
    }
    _helm(inputSource, source) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            this._logger.info("[_helm] path: %s, file: %s", inputSource.path, inputSource.file);
            let helmChartName = "";
            let helmChartPath = "";
            if (inputSource.kind === input_source_1.InputSourceKind.file) {
                try {
                    const contents = yield this._manifestsLoader.rawReadFile(source.id.path);
                    const chart = yaml_1.default.parseDocument(contents).toJS({});
                    helmChartName = chart.name;
                }
                catch (reason) {
                    this._logger.info("[_helm] ERROR: ", reason);
                    source.reportError('Failed to load Helm Chart manifest. Reason: ' + ((_a = reason === null || reason === void 0 ? void 0 : reason.message) !== null && _a !== void 0 ? _a : "Unknown"));
                    return;
                }
                if (!helmChartName) {
                    source.reportError('Invalid Helm Chart manifest. Chart name not set.');
                    return;
                }
                helmChartPath = path_1.default.dirname(inputSource.path);
            }
            else if (inputSource.kind === input_source_1.InputSourceKind.helm) {
                helmChartPath = inputSource.path;
            }
            const isLocalChart = fs_1.default.existsSync(helmChartPath);
            if (source.id.kind !== 'helm') {
                source = source.getSource("helm", helmChartPath, source.originalSource, !isLocalChart);
            }
            try {
                let command = `helm template`;
                for (const suffix of inputSource.suffixes) {
                    if (suffix.key === 'values') {
                        command += ` --values ${suffix.value}`;
                    }
                    else if (suffix.key === 'namespace') {
                        command += ` --namespace ${suffix.value}`;
                    }
                    else if (suffix.key === 'release-name') {
                        command += ` --release-name ${suffix.value}`;
                    }
                    else {
                        source.reportError(`Unknown preprocessor key provided: ${suffix.key}`);
                    }
                }
                if (source.selfErrors.length > 0) {
                    return;
                }
                command += ` ${helmChartPath}`;
                const contents = yield yield this.executeCommand(command);
                if (!contents) {
                    source.reportError(`Could not extract manifest contents from ${inputSource.preprocessor}`);
                }
                else {
                    try {
                        const rawYamls = this._manifestsLoader.parseYamlRaw(contents);
                        for (const rawYaml of rawYamls) {
                            this._processHelm(inputSource, source, rawYaml, helmChartName, isLocalChart);
                        }
                    }
                    catch (reason) {
                        source.reportError(`Failed to parse YAML manifests from ${inputSource.preprocessor}. Reason: ${(_b = reason === null || reason === void 0 ? void 0 : reason.message) !== null && _b !== void 0 ? _b : "Unknown"}`);
                        return;
                    }
                }
            }
            catch (reason) {
                this._logger.info("[_helm] ERROR: ", reason);
                source.reportError(`Failed to extract manifest from ${inputSource.preprocessor}. Reason: ${(_c = reason === null || reason === void 0 ? void 0 : reason.message) !== null && _c !== void 0 ? _c : "Unknown"}`);
            }
        });
    }
    _processHelm(inputSource, source, rawYaml, helmChartName, isLocalChart) {
        var _a;
        const comment = (_a = rawYaml.contents) === null || _a === void 0 ? void 0 : _a.commentBefore;
        this._logger.info("[_processHelm] comment: %s", comment);
        if (comment) {
            const matches = comment.match(/Source: (\S*)$/);
            if (matches) {
                let templatePath = matches[1];
                this._logger.info("[_processHelm] templatePath: %s", templatePath);
                if (inputSource.kind === input_source_1.InputSourceKind.file) {
                    if (helmChartName && helmChartName.length > 0) {
                        if (the_lodash_1.default.last(helmChartName) !== '/') {
                            helmChartName += '/';
                        }
                        if (templatePath.startsWith(helmChartName)) {
                            templatePath = templatePath.substring(helmChartName.length);
                        }
                    }
                }
                source = source.getSource('file', templatePath, source.originalSource, !isLocalChart);
            }
        }
        const config = this._manifestsLoader.rawYamlToObj(rawYaml);
        this._manifestsLoader.addManifest(source, config);
    }
    executeCommand(command) {
        return __awaiter(this, void 0, void 0, function* () {
            this._logger.info("EXECUTING: %s", command);
            const spinner = (0, spinner_1.spinOperation)(`Executing ${command}`);
            try {
                const result = yield the_promise_1.Promise.construct((resolve, reject) => {
                    (0, child_process_1.exec)(command, (err, output) => {
                        // once the command has completed, the callback function is called
                        if (err) {
                            // log and return if we encounter an error
                            this._logger.error("ERROR: %s", output);
                            reject(err);
                            return;
                        }
                        // log the output received from the command
                        // this._logger.info("Output: %s", output)
                        resolve(output);
                    });
                });
                spinner.complete(`Succeeded: ${command}`);
                return result;
            }
            catch (reason) {
                spinner.fail(`Command Failed: ${command}`);
                throw reason;
            }
            return null;
        });
    }
}
exports.PreProcessorExecutor = PreProcessorExecutor;
//# sourceMappingURL=executor.js.map